﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using XrMapLib;

namespace K_Means
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            //Xr.MapBackgoundColor = Color.Black;
        }

        private void btnAddData_Click(object sender, EventArgs e)
        {
            DialogResult DR = openFileDialog.ShowDialog();

            if (DR == DialogResult.OK)
            {
                Xr.Layers.RemoveAllLayers();

                String FileName = openFileDialog.FileName;
                bool bOK = Xr.Layers.AddShapeMapLayer("lyr", FileName);
                if(bOK)
                {
                    Xr.Layers.AddGraphicLayer("gl", "");
                    Xr.WaitForAllConnections();
                    Xr.ZoomFullExtent();
                    Xr.Update();
                }
            }
        }

        private uint PastelColor(Random Rnd)
        {
            Byte r = (Byte)(Math.Round(Rnd.NextDouble() * 206) + 48);
            Byte g = (Byte)(Math.Round(Rnd.NextDouble() * 206) + 48);
            Byte b = (Byte)(Math.Round(Rnd.NextDouble() * 206) + 48);

            return RGB(b, g, r);
        }

        private uint RGB(Byte R, Byte G, Byte B)
        {
            return (uint)ColorTranslator.ToOle(Color.FromArgb(R, G, B));
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            Random Rnd = new Random();

            int K = int.Parse(tbK.Text);
            ShapeTable ShapeTable = Xr.Layers.GetLayerAsShapeMap("lyr").ShapeTable;
            KMeans KMeans = new KMeans(K, ShapeTable);

            if (KMeans.Run())
            {
                Dictionary<int, KMeans.Cluster> Clusters = KMeans.Clusters;
                int cntClusters = Clusters.Count;

                Xr.Layers.RemoveLayer("gl");
                Xr.Layers.AddGraphicLayer("gl", "");
                Xr.WaitForAllConnections();

                for (int iCluster=0; iCluster < cntClusters; iCluster++)
                {
                    KMeans.Cluster Cluster = Clusters[iCluster];
                    LinkedList<int> Members = Cluster.Members;
                    int cntRows = Members.Count;

                    ConvexHull.Point[] pts = new ConvexHull.Point[cntRows];
                    LinkedList<int>.Enumerator Iter = Members.GetEnumerator();
                    int i = 0;
                    while(Iter.MoveNext()) 
                    {
                        int FID = (int)Iter.Current;
                        XrMapLib.ShapeRow row = ShapeTable.GetRow(FID);
                        if (row.Load())
                        {
                            pts[i++] = new ConvexHull.Point(row.Centroid.X, row.Centroid.Y);
                            row.Unload();
                        }
                    }

                    pts = ConvexHull.Get(pts);

                    XrMapLib.IPolygonGraphicElement gr = new XrMapLib.PolygonGraphicElement();

                    XrMapLib.CoordListCollection clc = new XrMapLib.CoordListCollection();
                    XrMapLib.CoordList cl = new XrMapLib.CoordList();

                    for (int iPt = 0; iPt < pts.Length; iPt++)
                    {
                        cl.Add(pts[iPt].x, pts[iPt].y);
                    }

                    clc.Add(cl);

                    gr.SetPolygonList(clc);

                    gr.FillSymbol.Color = PastelColor(Rnd);
                    gr.FillSymbol.Alpha = 200;
                    gr.LineSymbol.Alpha = 0;

                    XrMapLib.IGraphicLayer gl = Xr.Layers.GetLayerAsGraphic("gl");

                    gl.AddElement(iCluster, gr as XrMapLib.GraphicElement);
                }

                Xr.Update();
            }
        }
    }
}
