﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XrMapLib;
using System.Windows.Forms;

namespace K_Means
{
    public class KMeans
    {
        public class Cluster {
            private double X;
            private double Y;

            private double SumX;
            private double SumY;

            public LinkedList<int> Members = null;

            public Cluster(double X, double Y)
            {
                Members = new LinkedList<int>();

                this.X = X;
                this.Y = Y;

                this.SumX = 0;
                this.SumY = 0;
            }

            public Cluster(Cluster C)
            {
                Members = new LinkedList<int>();

                this.X = C.X;
                this.Y = C.Y;

                this.SumX = 0;
                this.SumY = 0;
            }

            public double Distance(double X, double Y)
            {
                return Math.Sqrt((this.X - X) * (this.X - X) + (this.Y - Y) * (this.Y - Y));
            }

            public void Add(int FID, double X, double Y)
            {
                Members.AddFirst(FID);
                SumX += X;
                SumY += Y;
            }

            public bool RegenXY()
            {
                double NewX = SumX / Members.Count;
                double NewY = SumY / Members.Count;

                bool Result = (NewX != X) || (NewY != Y);

                X = NewX;
                Y = NewY;

                return Result;
            }
        }

        public Dictionary<int, Cluster> Clusters = null;
        private int K;
        private ShapeTable ShapeTable;

        public KMeans(int K, ShapeTable Tbl)
        {
            this.K = K;
            this.ShapeTable = Tbl;

            Clusters = new Dictionary<int, Cluster>();

            if (Tbl.RowCount < K) return;

            for(int i = 0; i < K; i++)
            {
                ShapeRow Row = Tbl.GetRow(i);
                if(Row.Load())
                {
                    Coord Pt = Row.Centroid;
                    Clusters.Add(i, new Cluster(Pt.X, Pt.Y));

                    Row.Unload();
                }
            }
        }

        public bool Run()
        {
            int cntRows = ShapeTable.RowCount;
            bool bGoOn = true;

            while (bGoOn)
            {
                Dictionary<int, Cluster> NewClusters = new Dictionary<int, Cluster>();

                for (int i = 0; i < K; i++)
                {
                    Cluster Cluster = Clusters[i];
                    NewClusters.Add(i, new Cluster(Cluster));
                }

                for (int FID = 0; FID < cntRows; FID++)
                {
                    ShapeRow Row = ShapeTable.GetRow(FID);
                    if (Row.Load())
                    {
                        Coord Pt = Row.Centroid;
                        double MinDist = Double.MaxValue;
                        int iTagretCluster = 0;
                        for (int iCluster = 0; iCluster < K; iCluster++)
                        {
                            Cluster Cluster = NewClusters[iCluster];

                            double Dist = Cluster.Distance(Pt.X, Pt.Y);
                            if (MinDist > Dist)
                            {
                                MinDist = Dist;
                                iTagretCluster = iCluster;
                            }
                        }

                        NewClusters[iTagretCluster].Add(FID, Pt.X, Pt.Y);

                        Row.Unload();
                    }
                }

                bGoOn = false;
                for (int i = 0; i < K; i++)
                {
                    Cluster Cluster = NewClusters[i];
                    if (Cluster.RegenXY())
                    {
                        bGoOn = true;
                    }
                }

                Clusters = NewClusters;
            }

            return true;
        }
    }
}
